/* create the table */
Create Table Record_IDs(
	Table_Name  varchar(30),
	Current_ID  int)
	Go

 /* add a primary clustered index */
 Create      Unique Clustered Index PK_Record_IDs
       on Record_IDs( Table_Name ) with FILLFACTOR = 1
 Go
 Insert Record_IDs
       Select   Name,     1
       From   Sysobjects
       Where  Type = 'U' /* user defined tables */
 Create Procedure up_GetID                 /* up = user procedure */
       @psTableName      varchar(30),            /* p = parameter */
       @rnNewID    int OUTPUT        /* r = receive or output parameter */
 As
 Declare
       @nSQLError int,
       @nRowCount int
 
 Begin Tran
       /* First update the record to acquire the exclusive lock on the page */
       Update Record_IDs
       Set    Current_ID = Current_ID + 1
       Where  Table_Name = @psTableName
 
       /* Check for errors */
       Select    @nSQLError = @@error,
             @nRowCount = @@rowcount
       If @nSQLError != 0 OR @nRowCount != 1
       Begin
             Rollback Tran
             Return -999 /* failed to update record correctly */
       End
 
       /* Select back the value from the table that we've already locked */
       Select      @rnNewID = Current_ID
       From   Record_IDs
       Where  Table_Name = @psTableName
 
       /* Check for errors */
       Select     @nSQLError = @@error,
             @nRowCount = @@rowcount
       If @nSQLError != 0 OR @nRowCount != 1
       Begin
             Rollback Tran
             Return [nd]998 /* failed to select record correctly */
       End
 Commit Tran
 Return 0
 Go
 Declare
       @nRecordID  int,
       @nRC        int,
       @sMsg       varchar(255)
 
 /* Fetch a record ID for use in inserting new record */
 Exec @nRC = up_GetID 'table_A', @nRecordID OUTPUT
 
 If @nRC != 0
       Print 'An error occurred fetching new Record ID'
 Else
 Begin
       Select @sMsg = 'New Record value is ' + Convert( varchar(4), @nRecordID )
       Print @sMsg
 End
 Go

